; inc_ListViews.ahk
; Contains subroutines, etc. related to file lists and their associated events

;----- Editables list -----
BuildEditableList:
; Gather a list of editable file names from the current working directory folder.
	Gui, ListView, EditableListView
	LV_Delete()							; clear ListView
	Loop, %A_WorkingDir%\*.* 			; iterate on directory
	{
		gTempFileName = %A_LoopFileName%
		SplitPath, gTempFileName,,, fileExt	; Get the file's extension.
		if fileExt in %kEditableExts%
			LV_Add("", gTempFileName)
	}
	;activateCompileBtn(0)
	return

EditableList_event:
; used when user clicks on an entry in the editable filelist and for other list view events
	Gui +OwnDialogs
	Gui, ListView, EditableListView
	LV_GetText(gTempFileName, A_EventInfo)		; Get the row's first-column text.
	if A_GuiEvent = DoubleClick
	{
		if gTempFileName <> %kEditableHeader% 	; suppress clicks in blank areas
			gosub EditableList_eventDblClk

	}
	else if A_GuiEvent = RightClick
	{
		if gTempFileName <> %kEditableHeader% 	; suppress clicks in blank areas
		{
			Menu, EditListContextMenu, Add, bogus, DefaultAction	; add bogus menuy entry to make sure it exists
			Menu, EditListContextMenu, DeleteAll	; delete all entries and rebuild
			Menu, EditListContextMenu, Add, %kEditContextOpen%   %gTempFileName%, FileListContextOpen
			Menu, EditListContextMenu, Add, %kEditContextRename%, FileListContextRename
			SplitPath, gTempFileName,,, fileExt	; Get the file's extension.
			if fileExt in %kCompilableExts%
				Menu, EditListContextMenu, Add, %kEditContextCompile%, FileListContextCompile
			Menu, EditListContextMenu, Add
			Menu, EditListContextMenu, Add, %kEditContextDelete%, FileListContextDelete
			Menu, EditListContextMenu, Show, %A_GuiX%, %A_GuiY%
		}
		else
		{   Menu, EditListContextMenu2, Add, %kEditContextNew%, FileListContextNew
			Menu, EditListContextMenu2, Show, %A_GuiX%, %A_GuiY%
		}
	}
	return

EditableList_eventDblClk:
; If gTempFileName is editable, issue a dialog prompting the use to select Edit or Compile,
; otherwise Edit gTempFileName
	Gui +OwnDialogs
	gTempFileName_editlistdblclk = %gTempFileName%	; buffer gTempFileName in case it gets lost before MsgBox is dismissed
	SplitPath, gTempFileName_editlistdblclk,,, fileExt	; Get the file's extension.
	if fileExt in %kCompilableExts%		; if compilable issue "Edit or Compile?" dialog box
	{
		; TODO: figure out this GUI owner stuff to try to get the msgbox to behave more nicely
		boxTitle=%kEditOrCompileTitle%
		boxPrompt=%kEditOrCompileText%`n"%gTempFileName_editlistdblclk%"?
		boxOptions=Buttons%kEditOrCompile_editBtnTxt%|%kEditOrCompile_compileBtnTxt%|%kEditOrCompile_cancelBtnTxt% Default1 +AlwaysOnTop Icon? bw80 -Owner +Toolwindow
		gMsgBoxOption := Msgbox_mfk(boxPrompt,boxOptions,boxTitle)
		if (gMsgBoxOption="Edit")
		{
			gTempFileName = %gTempFileName_editlistdblclk%	; reset gTempFileName to stored value
			gosub EditableList_eventOpen
		}
		else if (gMsgBoxOption="Compile")
		{
			gTempFileName = %gTempFileName_editlistdblclk%	; reset gTempFileName to stored value
			gosub CompileTempFile
		}
	}
	else
	{
		gTempFileName = %gTempFileName_editlistdblclk%		; reset gTempFileName to stored value
		gosub EditableList_eventOpen
	}
	return

EditableList_eventOpen:
; minimizes main gui window and opens gTempFileName in external editor.
; caller must ensure that gTempFileName is valid
	; Check if the last character of the folder name is a backslash, which happens for root
	; directories such as C:\. If it is, remove it to prevent a double-backslash later on.
	gTempDirName = %A_WorkingDir%				; make local copy of A_WorkingDir
	StringRight, LastChar, gTempDirName, 1
	if LastChar = \
		StringTrimRight, gTempDirName, gTempDirName, 1	; Remove the trailing backslash.
	launchInEditor( """" . gTempDirName . "\" . gTempFileName . """" )	; concatenate "%gTempDirName%\%gTempFileName%"
	return

;----- Compiled files list -----
BuildCompiledList:
; Gather a list of compiled file names from the current working directory folder.
	Gui, ListView, CompiledListView
	LV_Delete()								; clear ListView
	Loop, %A_WorkingDir%\*.* 				; iterate on directory
	{
		gTempFileName = %A_LoopFileName%
		SplitPath, gTempFileName,,, fileExt	; Get the file's extension.
		if fileExt in %kCompiledExts%
			LV_Add("", gTempFileName)
	}
	return

CompiledList_event:
; used when user clicks on an entry in the editable filelist and for other list view events
	Gui +OwnDialogs
	Gui, ListView, CompiledListView
	if A_GuiEvent = DoubleClick
	{
		LV_GetText(gTempFileName, A_EventInfo)		; Get the row's first-column text.
		if gTempFileName <> %kCompiledHeader% 	; suppress clicks in blank areas
			gosub CompiledList_eventRun
	}
	else if A_GuiEvent = RightClick
	{
		LV_GetText(gTempFileName, A_EventInfo)		; Get the row's first-column text.
		if gTempFileName <> %kCompiledHeader% 	; suppress clicks in blank areas
		{
			Menu, CompiledListContextMenu, Add, bogus, DefaultAction	; add bogus menuy entry to make sure it exists
			Menu, CompiledListContextMenu, DeleteAll	; delete all entries and rebuild
			Menu, CompiledListContextMenu, Add, %kEditContextRun%	%gTempFileName%, FileListContextRun
			Menu, CompiledListContextMenu, Add, %kEditContextRename%, CompListContextRename
			Menu, CompiledListContextMenu, Add
			Menu, CompiledListContextMenu, Add, %kEditContextDelete%, FileListContextDelete
			Menu, CompiledListContextMenu, Show, %A_GuiX%, %A_GuiY%
		}
	}
	return

CompiledList_eventRun:
; Executes gTempFileName (assumed to be in A_WorkingDir), propting for command line arguments
; TODO: make this a function before side really effects mess up something!!!
	CompiledList_eventRun_fileName = %gTempFileName%	; buffer gTempFileName
	Gui +OwnDialogs
	InputBox, gArgList, %kRunTitle% "%CompiledList_eventRun_fileName%", %kRunPrompt%,,kNominalDialogW,kNominalDialogH,,,,, %gArgList%
	if ErrorLevel = 0 	; if we didn't cancel
	{
        SplitPath, gTempFileName,,,,className ; get the classname from gTempFileName
		; determine path to class from root, i.e. rootDir - workingDir)
		StringReplace, pathToClass, A_WorkingDir, %gRootDir%,,All
		; if pathToClass is not blank, then we need to add package names to className (e.g., com.foo.myClass)
		if pathToClass
		{
			; the first char of pathToClass may be a backslash... if so trim it
			firstChar := SubStr(pathToClass,1,1) 
			if firstChar = \
			{
				StringTrimLeft, pathToClass, pathToClass, 1
			}
			; now change all remaining backslashes to periods
			StringReplace, packageNames, pathToClass, \,.,All
			; now add the packageNames to className to form a full class name (e.g., com.foo.myClass)
			className = %packageNames%.%className%
		}
		; create a batch file in the working directory with the correct executable instructions and run it.
        FileDelete, %kTempBat%    ; first delete the old batch file in case it didn't get deleted for some reason
        FileAppend,               ; now make the new batch file
(
@echo  off
REM This is a temporary file. Delete this file if it has not been deleted automatically.
title %className%
REM "%gToolPath%\%kInterpreterTool%" -cp . %className% %gArgList%
REM "%gToolPath%\%kInterpreterTool%" -cp "." %className% %gArgList%
"%gToolPath%\%kInterpreterTool%" -classpath "%gRootDir%" %className% %gArgList%
echo.
echo.
pause
), %kTempBat%
        RunWait, %kTempBat%      ; run the batch file
        FileDelete, %kTempBat%   ; delete the batch file
	}
	gTempFileName = %CompiledList_eventRun_fileName%	; reset gTempFileName
	return

;----- Lists' Context Menus -----
FileListContextOpen:
	gosub EditableList_eventOpen
	return

CompListContextRename:
	isCompiledFile = 1 	; set compiled flag ... means we are trying to rename a compiled file
FileListContextRename:
;List_eventRename:
; prompts for and renames gTempFileName
	Gui +OwnDialogs
	strNewFileName = %gTempFileName%	; buffer gTempFileName
	InputBox, newEditFileName, %kRenameTitle% "%strNewFileName%", %kNewFileTxt%,, %kNominalDialogW%, %kNominalDialogH%,,,,, %strNewFileName%
	; check for errors
	if ErrorLevel <> 0		; if we canceled
	{
		isCompiledFile = 0		 	; reset compiled file flag
		return				; get out of here
		;newEditFileName =	; should we reset newEditFileName?
	}
	if !newEditFileName		; if we didn't get a filename
	{
		MsgBox, 48, %kRenameNoFileNameTitle%, %kRenameNoFileNameTxt%		; IconExclamation
		isCompiledFile = 0		 	; reset compiled file flag
		return				; get out of here
	}
	SplitPath, newEditFileName,,, fileExt	; Get the file's extension.
	if isCompiledFile							; if it's a compiled file ...
	{
		if fileExt not in %kCompiledExts%
		{
			MsgBox, 16, %kWrongExtensionTitle%, "%fileExt%" %kWrongExtensionTxt% %kCompiledExts%	; IconError
			gTempFileName = %strNewFileName%	; reset gTempFileName
			gosub CompListContextRename			; recursive fix condition
			isCompiledFile = 0		 					; reset compiled file flag
			return
		}
	}
	else								; if it's an editable...
	{
		if fileExt not in %kEditableExts%
		{
			MsgBox, 16, %kWrongExtensionTitle%, "%fileExt%" %kWrongExtensionTxt% %kEditableExts%	; IconError
			gTempFileName = %strNewFileName%	; reset gTempFileName
			gosub FileListContextRename			; recursive fix condition
			isCompiledFile = 0		 					; reset exe flag
			return
		}
	}
	IfExist, %newEditFileName%		;if newEditFileName already exists
	{
		MsgBox, 308, %kRenameTitle%, %kFileExists1% "%newEditFileName%" %kFileExists2%	; Yes/No+IconExclamation+2ndBtnDefault+SystemModal
		IfMsgBox, No				; if no...
		{
			gTempFileName = %strNewFileName%	; reset gTempFileName
			gosub FileListContextRename			; recursive fix condition (isCompiledFile flag is not reset at this point)
			isCompiledFile = 0		 					; reset compiled file flag
			return
		}
	}
	; if you made it here, there are no errors -- rename the file
	FileMove, %strNewFileName%, %newEditFileName%, 1	; move the file
	if isCompiledFile
		gosub BuildCompiledList
	else
		gosub BuildEditableList
	isCompiledFile = 0					 	; reset exe flag
	return

FileListContextCompile:
	gosub CompileTempFile
	return

FileListContextRun:
	gosub CompiledList_eventRun
	return

FileListContextDelete:
	gosub DeletegTempFileName
	return

FileListContextNew:
	gosub CreateNewFile
	return

;*****************************************;
;*************** Functions ***************;
;*****************************************;

